// (c) Copyright 2011-2012 Cloudera, Inc. All rights reserved.
define([
	"cloudera/chart/TimeRange",
	"cloudera/chart/ActivityChartSet",
	"cloudera/Util",
	"cloudera/chart/ChartUtil"
], function(TimeRange, ActivityChartSet, Util, ChartUtil) {

/**
 * This represents one or more activity charts on the monitor activities page.
 */
return ActivityChartSet.extend({

	// TODO:
	// make the constant here a common constants else where.
	_broadcastRangeChange: function() {
		jQuery.publish("chartRangeChanged", [new TimeRange(this.firstVisibleDate, this.lastVisibleDate)]);
	},

	initializeTime: function() {
		this._super.apply(this, arguments);
		var self = this;

		var updateSelectedRange = function(range, isCurrentMode){
			// the chart's range has changed, update my highlighted selection.
			ChartUtil.updateOnTimeSelectionChange(self, range, isCurrentMode);
		};
		jQuery.subscribe("timeSelectionChanged", updateSelectedRange);

		/**
		 * When the context tree table is refreshed,
		 * we need to hide the showChart icons for those
		 * entries that have already been selected.
		 */
		var onContextTreeTableUpdated = function(){
			var result = [];

			var $selectedContexts = $(".selectedContexts").find(".selectedContext");
			for (j = 0; j < $selectedContexts.length; j += 1) {
				var $selectedContext = $selectedContexts[j];
				var selectedContextData = $.data($selectedContext, 'context');
				if (selectedContextData) {
					result.push(selectedContextData);
				}
			}
			jQuery.publish("contextSelectionAdded", [result]);
		};

		/**
		 * When we add a context, also add its label to the toolbar area.
		 */
		var addLabelForContexts = function(contexts) {
			var i;
			for (i = 0; i < contexts.length; i += 1) {
				var context = contexts[i];
				if (!findContext(context)) {

					// Create a span.
					var $span = jQuery("<span class='selectedContext'/>");
					$span.addClass("activityType" + context.activityType);

					// An Icon
					var $icon = jQuery("<span class='icon'/>");

					// A link
					var $anchor = jQuery("<a/>");
					$anchor.attr("href", "/cmf/jobs/" + context.id + "/");
					$anchor.html(context.id);
					$anchor.attr("title", context.name);

					// A remove button
					var $removeButton = jQuery("<span class='removeContext IconButton ui-icon ui-icon-close' style='display:inline-block' title='Remove'></span>;");

					// Add it to the toolbar area.
					$span.append($icon);
					$span.append($anchor);
					$span.append($removeButton);
					jQuery.data($span[0], "context", context);
					jQuery(".selectedContexts").append($span);
				}
			}
		};

		/**
		 * @returns true if the context has been added before.
		 */
		var findContext = function(context) {
			var found, j, selectedContexts;
			selectedContexts = $(".selectedContexts").find(".selectedContext");
			for (j = 0; j < selectedContexts.length; j += 1) {
				var data = $.data(selectedContexts[j], 'context');
				if (data && data.id === context.id) {
					found = true;
					break;
				}
			}
			return found;
		};

		/**
		 * When the selected context area is clicked,
		 * check if the remove icon is clicked.
		 */
		var onSelectedContextsAreaClicked = function(evt) {
			var $rem  = $(evt.target);
			if ($rem.is(".removeContext")) {
				var $span = $($rem).closest(".selectedContext");
				if ($span.length > 0) {
					var context = $.data($span[0], "context");
					if (context) {
						var contexts = [context];
						jQuery($span[0]).remove();
						self.removeContexts(contexts);
						jQuery.publish("contextSelectionRemoved", [contexts]);
					}
				}
			}
		};

		/**
		 * Handles when the cluster checkbox is clicked.
		 */
		var onShowHideClusterCheckboxClicked = function(evt) {
			var contexts = [{
				id: "cluster",
				name: "cluster",
				type: "CLUSTER"
			}];
			if ($(evt.target).prop("checked")) {
				self.addContexts(contexts);
			} else {
				self.removeContexts(contexts);
			}
		};

		$(".selectedContexts")
			.click(onSelectedContextsAreaClicked);

		$(".hideShowCluster")
			.click(onShowHideClusterCheckboxClicked)
			.change(onShowHideClusterCheckboxClicked);

		jQuery.subscribe("contextTreeTableUpdated", onContextTreeTableUpdated);

		jQuery.subscribe("setChartDescriptors", function(chartDescriptors) {
			var comparator = function(m1, m2) {
				return (m1.metric.id === m2.metric.id && m1.metric.context === m2.metric.context);
			};
			var addRemoves = ChartUtil.getAddRemoves(self.chartDescriptors, chartDescriptors, comparator);
			var skipRender = true;
			self.removeChartDescriptors(addRemoves.removes, skipRender);
			self.addChartDescriptors(addRemoves.adds, skipRender);
			self.render();
		});

		jQuery.subscribe("addContexts", function(contexts) {
			self.addContexts(contexts);
			addLabelForContexts(contexts);
		});

		jQuery.subscribe("removeContexts", function(contexts) {
			self.removeContexts(contexts);
			// TODO: add removeLabelForContexts.
			// Currently, the only way to remove is from the toolbar.
		});
	},

	onChartDraw: function() {
		this._super.apply(this, arguments);
		this.zoomTo(this.firstVisibleDate, this.lastVisibleDate);
		ChartUtil.resize(this, this.options.chartWidth, this.options.chartHeight);
	}
});

});
