// (c) Copyright 2011-2012 Cloudera, Inc. All rights reserved.
define([
	"cloudera/Util"
], function(Util) {

	/**
	 * Updates Heatmap Charts. It listens to
	 * markerDateChanged and heatmapChartChanged events.
	 */
	var HeatmapChartUpdater = function(options) {
		var ajaxUrl = options.ajaxUrl;
		var containerSelector = options.containerSelector;
		var oldMarkerDate = new Date(options.timestamp);
		var oldCurrentMode = options.currentMode;

		var oldTestName = "";
		var oldChartId = "";
		var oldSelectedCellId;

		var prepareUrl = function () {
			var urlParams = {};
			if (Util.isNumber(oldMarkerDate.getTime())) {
				urlParams.timestamp = oldMarkerDate.getTime();
			} else {
				urlParams.timestamp = Util.getServerNow().getTime();
			}
			urlParams.currentMode = oldCurrentMode;
			urlParams.testName = oldTestName;
			urlParams.chartId = oldChartId;
			if (options.key) {
				urlParams.key = options.key;
			}

			if (oldSelectedCellId) {
				urlParams.selectedCellId = oldSelectedCellId;
			}

			if (oldTestName !== "" && oldChartId !== "") {
				var url = ajaxUrl + "?" + $.param(urlParams);
				$.post(url, function(response) {
					var filteredResponse = Util.filterError(response);
					$(containerSelector).empty().html(filteredResponse);
				});
			}
		};

		var onMarkerDateChanged = function (markerDate, currentMode) {
			if (Util.compareDates(oldMarkerDate, markerDate)) {
				oldMarkerDate = markerDate;
				oldCurrentMode = currentMode;
				updateIfVisible();
			}
		};

		var onChartChanged = function (testName, chartId) {
			if (oldTestName !== testName || oldChartId !== chartId) {
				oldTestName = testName;
				oldChartId = chartId;
				updateIfVisible();
			}
		};

		var updateIfVisible = function () {
			if ($(containerSelector).is(":visible")) {
				prepareUrl();
			}
		};

		var onSelectionChanged = function (id) {
			oldSelectedCellId = id;
		};

		jQuery.subscribe("markerDateChanged", onMarkerDateChanged);
		jQuery.subscribe("heatmapChartChanged", onChartChanged);
		jQuery.subscribe("heatmapCellSelectionChanged", onSelectionChanged);

		$('body').on('click', "[class~=heatmapChartLink]", function (evt) {
			$(".heatmapChartLink").closest("li").removeClass("active");
			var $target = $(evt.target);
			$target.closest("li").addClass("active");
			var newChartId = $target.attr("data-chart-id");
			var newTestName = $target.attr("data-test-name");
			$.publish("heatmapChartChanged", [newTestName, newChartId]);
		});
	};

	return HeatmapChartUpdater;
});
